@@ -1,17 +1,16 @@
 # SVG Schematics by Ken Kundert
 # encoding: utf8
 
-
 # Description {{{1
 """
 *SVG Schematic* is a Python library can be used to create schematics using SVG.
 """
-__version__ = '1.0.0'
-__released__ = '2020-04-16'
+__version__ = '1.2.0'
+__released__ = '2022-06-03'
 
 
 # License {{{1
-# Copyright (C) 2018-20 Kenneth S. Kundert
+# Copyright (C) 2018-2022 Kenneth S. Kundert
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -71,6 +70,46 @@ def with_y(p1, a2):
         # second argument is a number, use it
         return (p1[0], a2)
 
+# with_min_x() {{{2
+def with_min_x(p1, *args):
+    """Returns the first argument (a coordinate pair) with the x value replaced with
+    the smallest x value of the remaining arguments."""
+    if args:
+        min_x = min(a[0] if isinstance(a, tuple) else a for a in args)
+    else:
+        min_x = p1[0]
+    return (min_x, p1[1])
+
+# with_max_x() {{{2
+def with_max_x(p1, *args):
+    """Returns the first argument (a coordinate pair) with the x value replaced with
+    the largest x values of the remaining arguments."""
+    if args:
+        max_x = max(a[0] if isinstance(a, tuple) else a for a in args)
+    else:
+        max_x = p1[0]
+    return (max_x, p1[1])
+
+# with_min_y() {{{2
+def with_min_y(p1, *args):
+    """Returns the first argument (a coordinate pair) with the y value replaced with
+    the smallest y value of the remaining arguments."""
+    if args:
+        min_y = min(a[1] if isinstance(a, tuple) else a for a in args)
+    else:
+        min_y = p1[1]
+    return (p1[0], min_y)
+
+# with_max_y() {{{2
+def with_max_y(p1, *args):
+    """Returns the first argument (a coordinate pair) with the y value replaced with
+    the largest y values of the remaining arguments."""
+    if args:
+        max_y = max(a[1] if isinstance(a, tuple) else a for a in args)
+    else:
+        max_y = p1[1]
+    return (p1[0], max_y)
+
 # midpoint() {{{2
 def midpoint(p1, p2):
     """Returns the point midway between two points."""
@@ -78,12 +117,14 @@ def midpoint(p1, p2):
 
 # midpoint_x() {{{2
 def midpoint_x(p1, p2):
-    """Returns the point midway between two points."""
+    """Returns the point with x value midway between two points
+    and the y value of the first point."""
     return ((p1[0] + p2[0])/2, p1[1])
 
 # midpoint_y() {{{2
 def midpoint_y(p1, p2):
-    """Returns the point midway between two points."""
+    """Returns the point with y value midway between two points
+    and the y value of the first point."""
     return (p1[0], (p1[1] + p2[1])/2)
 
 # offsets_to_coordinates() {{{2
@@ -360,6 +401,8 @@ class Tile(Schematic): # {{{1
         self.pins = {k: (w*v[0], h*v[1]) for k, v in pins.items()}
 
         # rotate and flip pins about normalized center
+        # rotate (v or h) is the rotation that requires rotation if requested.
+        # Thus if a symbol is drawn horizontally, then rotate would be 'v'
         pins = {}
         for name, loc in self.pins.items():
             x, y = loc
@@ -629,6 +672,7 @@ class Inductor(Tile): # {{{1
         xinc = 20
         xdec = -5
         ypeak = 15
+        dy = 5  # extra vertical spacing for text
         ytrough = -10
         undulations = 4
         p = self.pins['p']
@@ -687,9 +731,9 @@ class Inductor(Tile): # {{{1
                 self.add_text(value, shift(self.center, x_nudge, nudge), 'u'+just)
         else:
             if name:
-                self.add_text(name, shift(self.center, 0, -ypeak), 'lm')
+                self.add_text(name, shift(self.center, 0, -ypeak-dy), 'lm')
             if value:
-                self.add_text(value, shift(self.center, 0, ypeak), 'um')
+                self.add_text(value, shift(self.center, 0, ypeak+dy), 'um')
 
 
 class Diode(Tile): # {{{1
@@ -1094,7 +1138,7 @@ class Amp(Tile): # {{{1
             Specify the offset from the specified location.
     '''
     def __init__(
-        self, kind='oa', orient='h', name=None, value=None, w=2, h=2, **kwargs
+        self, kind='se', orient='h', name=None, value=None, w=2, h=2, **kwargs
     ):
         # Initialization and parameters {{{2
         pins = dict(
@@ -1112,77 +1156,93 @@ class Amp(Tile): # {{{1
         w, h = self.size
         lw = schematic.sch_line_width
         sign_size = 14
-        nudge = 5
+        nudge = kwargs.get('nudge', 0)
+        delta = 5
 
-        # Amp {{{2
-        amp = schematic.polygon(
-            [   ( w/2,  0),
-                (-w/2, -h/2),
-                (-w/2, +h/2),
-            ], fill=schematic.sch_background,
-            stroke_width=lw, stroke='black', stroke_linecap='round'
-        )
-        symbol.add(amp)
+        # Draw symbol {{{2
+        if isinstance(self, Converter):
+            converter = schematic.polygon(
+                [   ( w/2,  0),
+                    (   0, -h/2),
+                    (-w/2, -h/2),
+                    (-w/2, +h/2),
+                    (   0, +h/2),
+                ], fill=schematic.sch_background,
+                stroke_width=lw, stroke='black', stroke_linecap='round'
+            )
+            out_x = w/4
+            symbol.add(converter)
+        else:
+            amp = schematic.polygon(
+                [   ( w/2,  0),
+                    (-w/2, -h/2),
+                    (-w/2, +h/2),
+                ], fill=schematic.sch_background,
+                stroke_width=lw, stroke='black', stroke_linecap='round'
+            )
+            out_x = 0
+            symbol.add(amp)
         if kind in 'oa da comp'.split():
             minus = schematic.line(
-                start=(-w/2+nudge, h/4),
-                end=(-w/2+nudge+sign_size, h/4),
+                start=(-w/2+delta, h/4),
+                end=(-w/2+delta+sign_size, h/4),
                 stroke_width=lw, stroke='black', stroke_linecap='round'
             )
             symbol.add(minus)
             plus_ew = schematic.line(
-                start=(-w/2+nudge, -h/4),
-                end=(-w/2+nudge+sign_size, -h/4),
+                start=(-w/2+delta, -h/4),
+                end=(-w/2+delta+sign_size, -h/4),
                 stroke_width=lw, stroke='black', stroke_linecap='round'
             )
             symbol.add(plus_ew)
             plus_ns = schematic.line(
-                start=(-w/2+nudge+sign_size/2, -h/4+sign_size/2),
-                end=(-w/2+nudge+sign_size/2, -h/4-sign_size/2),
+                start=(-w/2+delta+sign_size/2, -h/4+sign_size/2),
+                end=(-w/2+delta+sign_size/2, -h/4-sign_size/2),
                 stroke_width=lw, stroke='black', stroke_linecap='round'
             )
             symbol.add(plus_ns)
         if kind == 'da':
             minus = schematic.line(
-                start=(-2*sign_size+nudge, -h/4),
-                end=(-sign_size+nudge, -h/4),
+                start=(out_x-2*sign_size+delta, -h/4),
+                end=(out_x-sign_size+delta, -h/4),
                 stroke_width=lw, stroke='black', stroke_linecap='round'
             )
             symbol.add(minus)
             plus_ew = schematic.line(
-                start=(-2*sign_size+nudge, h/4),
-                end=(-sign_size+nudge, h/4),
+                start=(out_x-2*sign_size+delta, h/4),
+                end=(out_x-sign_size+delta, h/4),
                 stroke_width=lw, stroke='black', stroke_linecap='round'
             )
             symbol.add(plus_ew)
             plus_ns = schematic.line(
-                start=(-3*sign_size/2+nudge, h/4+sign_size/2-1),
-                end=(-3*sign_size/2+nudge, h/4-sign_size/2),
+                start=(out_x-3*sign_size/2+delta, h/4+sign_size/2-1),
+                end=(out_x-3*sign_size/2+delta, h/4-sign_size/2),
                 stroke_width=lw, stroke='black', stroke_linecap='round'
             )
             symbol.add(plus_ns)
             p_out_lead = schematic.line(
-                start=(0, h/4),
+                start=(out_x, h/4),
                 end=(w/2, h/4),
                 stroke_width=lw, stroke='black', stroke_linecap='round'
             )
             symbol.add(p_out_lead)
             n_out_lead = schematic.line(
-                start=(0, -h/4),
+                start=(out_x, -h/4),
                 end=(w/2, -h/4),
                 stroke_width=lw, stroke='black', stroke_linecap='round'
             )
             symbol.add(n_out_lead)
         if kind == 'comp':
             comp = schematic.polyline(
-                [   (w/8-nudge,  h/8),
-                    (w/8,        h/8),
-                    (w/8,       -h/8),
-                    (w/8+nudge, -h/8)
+                [   (-w/2 + w/8-delta,  h/8),
+                    (-w/2 + w/8,        h/8),
+                    (-w/2 + w/8,       -h/8),
+                    (-w/2 + w/8+delta, -h/8)
                 ],
                 fill='none',
                 stroke_width=lw, stroke='black', stroke_linecap='round'
             )
+            nudge += 10
             symbol.add(comp)
 
         # Orientation and translation {{{2
@@ -1202,12 +1262,15 @@ class Amp(Tile): # {{{1
                 nudge = -h/8 if '-' in orient else h/8
                 self.add_text(name, shift(self.center, 0, nudge), 'mm')
             else:
-                nudge = w/8 if '|' in orient else -w/8
+                nudge = w/8 - nudge if '|' in orient else -w/8 + nudge
                 self.add_text(name, shift(self.center, nudge, 0), 'mm')
         #if value:  ignore for now
         #    self.add_text(value, shift(self.center, -half/4, nudge), 'um')
 
 
+class Converter(Amp): # {{{1
+    pass
+
 class Gate(Tile): # {{{1
     '''Add gate to schematic.
 
@@ -1372,8 +1435,8 @@ class Source(Tile): # {{{1
 
     Args:
         kind (str): choose from:
-             'empty', 'vdc', 'idc', 'sine', 'sum', 'mult',
-            'cv' (controlled voltage) or 'ci' (controlled current)
+            'empty', 'vdc', 'idc', 'sine', 'sum', 'mult',
+            'cv' (controlled voltage), 'ci' (controlled current), or 'noise'
         orient (str): choose from:
             'v' = vertical,
             'h' = horizontal (default),
@@ -1698,7 +1761,7 @@ class Label(Tile): # {{{1
         color (str): color of the marker
         nudge (num): offset used when positioning text (if needed)
         C, N, NE, E, SE, S, SW, W, NW, p, n (xy location):
-            Use to specify the location of a feature of the resistor.
+            Use to specify the location of a feature of the label.
         off (xy location), xoff (real), yoff (real):
             Specify the offset from the specified location.
     '''
@@ -1805,8 +1868,8 @@ class Box(Tile): # {{{1
         nudge (num): offset used when positioning text (if needed)
         line_width (num): the line width
         background (str): color of interior of box
-        C, N, NE, E, SE, S, SW, W, NW, p, n (xy location):
-            Use to specify the location of a feature of the resistor.
+        C, N, NE, E, SE, S, SW, W, NW, i, pi, ni, o, po, (xy location):
+            Use to specify the location of a feature of the box.
         off (xy location), xoff (real), yoff (real):
             Specify the offset from the specified location.
     '''
@@ -1830,7 +1893,6 @@ class Box(Tile): # {{{1
         schematic = self.sch_schematic
         w, h = self.size
         lw = schematic.sch_line_width if line_width is None else line_width
-        terms = [(w/2, 0), (-w/2, 0)]
 
         # Box {{{2
         fill = schematic.sch_background if background is None else background
@@ -1878,7 +1940,7 @@ class Switch(Tile): # {{{1
         dots (bool): whether the dots should be drawn
         nudge (num): offset used when positioning text (if needed)
         C, N, NE, E, SE, S, SW, W, NW, p, n (xy location):
-            Use to specify the location of a feature of the resistor.
+            Use to specify the location of a feature of the switch.
         off (xy location), xoff (real), yoff (real):
             Specify the offset from the specified location.
     '''
@@ -2008,3 +2070,76 @@ class Switch(Tile): # {{{1
                 just = vjust + 'm'
             if name:
                 self.add_text(name, shift(self.center, -dx, nudge), just)
+
+
+class Crossing(Tile): # {{{1
+    '''Add wire crossing to schematic.
+
+    Args:
+        C, N, NE, E, SE, S, SW, W, NW, pi, ni, po, no (xy location):
+            Use to specify the location of a feature of the crossing.
+        off (xy location), xoff (real), yoff (real):
+            Specify the offset from the specified location.
+        orient (str): choose from ...
+            'v' = vertical,
+            'h' = horizontal (default),
+            '-' = flip about horizontal axis,
+            '|' = flip about vertical axis
+        pass_under (str): color of the pass under concealer
+
+    You will need to specify pass_under if the schematic background color is
+    none.  You can set it to 'none' to eliminate the pass under concealer.
+    '''
+
+    def __init__(self, orient='h', w=1, h=1, pass_under=None, **kwargs):
+        # Initialization and parameters {{{2
+        pins = dict(
+            pi = (-1/2, -1/2),
+            ni = (-1/2,  1/2),
+            po = ( 1/2,  1/2),
+            no = ( 1/2, -1/2),
+        )
+        self.set_coordinates(kwargs, pins, orient, 'v', w=w, h=h)
+        super().__init__()
+        symbol = self.symbol
+        schematic = self.sch_schematic
+        w, h = self.size
+        r = 3
+        lw = schematic.sch_line_width
+
+        # Concealer {{{2
+        # These are use to hide wiring that pass under component.
+        concealer = schematic.rect(
+            insert = (-w/2, -h/2),
+            size = self.size,
+            stroke = schematic.sch_background,
+            stroke_width = 2*lw,
+            fill = schematic.sch_background,
+        )
+        symbol.add(concealer)
+
+        # Crossing {{{2
+        # wire from upper left to lower right
+        wire = schematic.line(
+            start=(-w/2, h/2), end=(w/2, -h/2),
+            stroke_width=lw, stroke='black',
+        )
+        symbol.add(wire)
+
+        # pass under concealer (a white dot)
+        pass_under_concealer = schematic.circle(
+            (0, 0), r=r,
+            stroke = 'none',
+            fill = pass_under or schematic.sch_background
+        )
+        symbol.add(pass_under_concealer)
+
+        # wire from lower left to upper right
+        wire = schematic.line(
+            start=(-w/2, -h/2), end=(w/2, h/2),
+            stroke_width=lw, stroke='black',
+        )
+        symbol.add(wire)
+
+        # Orientation and translation {{{2
+        symbol.translate(self.center)
